@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 #
 # Version: 0.3.2
 #
@@ -18,6 +18,7 @@
 #   maxhostfileage=10
 #   maxthreads=15
 #   #exclude=.gov, facebook
+#   #prefer=your.favourite.mirror
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,6 +53,7 @@ requires_api_version = '2.5'
 plugin_type = (TYPE_CORE,)
 
 verbose = False
+always_print_best_host = True
 socket_timeout = 3
 timedhosts = {}
 hostfilepath = ''
@@ -59,6 +61,7 @@ maxhostfileage = 10
 loadcache = False
 maxthreads = 15
 exclude = None
+prefer = None
 downgrade_ftp = True
 done_sock_timeout = False
 done_repos = set()
@@ -87,14 +90,19 @@ def init_hook(conduit):
 
     """
     global verbose, socket_timeout, hostfilepath, maxhostfileage, loadcache
-    global maxthreads, exclude, downgrade_ftp
+    global maxthreads, exclude, prefer, downgrade_ftp
+    if hasattr(conduit, 'registerPackageName'):
+        conduit.registerPackageName("yum-plugin-fastestmirror")
     verbose = conduit.confBool('main', 'verbose', default=False)
+    always_print_best_host = conduit.confBool('main', 'always_print_best_host',
+                                              default=True)
     socket_timeout = conduit.confInt('main', 'socket_timeout', default=3)
     hostfilepath = conduit.confString('main', 'hostfilepath',
             default='/var/cache/yum/timedhosts')
     maxhostfileage = conduit.confInt('main', 'maxhostfileage', default=10)
     maxthreads = conduit.confInt('main', 'maxthreads', default=10)
     exclude = conduit.confString('main', 'exclude', default=None)
+    prefer = conduit.confString('main', 'prefer', default='no.prefer.mirror')
     downgrade_ftp = conduit.confBool('main', 'downgrade_ftp', default=True)
     # If the file hostfilepath exists and is newer than the maxhostfileage,
     # then load the cache.
@@ -130,7 +138,7 @@ def _len_non_ftp(urls):
         if url.startswith("ftp:"):
             continue
         num += 1
-    return 0
+    return num
 
 def postreposetup_hook(conduit):
     """
@@ -151,7 +159,7 @@ def postreposetup_hook(conduit):
     @param loadcache : Fastest Mirrors to be loaded from plugin's cache file or not.
     @type loadcache : Boolean
     """
-    global loadcache, exclude
+    global loadcache, exclude, prefer
 
     opts, commands = conduit.getCmdLine()
     if conduit._base.conf.cache or not _can_write_results(hostfilepath):
@@ -192,14 +200,19 @@ def postreposetup_hook(conduit):
         if str(repo) not in repomirrors:
             repomirrors[str(repo)] = FastestMirror(repo.urls).get_mirrorlist()
         if exclude:
-            for mirror in repomirrors[str(repo)]:
+            def excludeCheck(mirror):
                 if filter(lambda exp: exp in host(mirror),
                           exclude.replace(',', ' ').split()):
                     conduit.info(2, "Excluding mirror: %s" % host(mirror))
-                    repomirrors[str(repo)].remove(mirror)
+                    return False
+                return True
+            repomirrors[str(repo)] = filter(excludeCheck,repomirrors[str(repo)])
         repo.urls = repomirrors[str(repo)]
         if len(repo.urls):
-            conduit.info(2, " * %s: %s" % (str(repo), host(repo.urls[0])))
+            lvl = 3
+            if always_print_best_host:
+                lvl = 2
+            conduit.info(lvl, " * %s: %s" % (str(repo), host(repo.urls[0])))
         repo.failovermethod = 'priority'
         repo.check()
         repo.setupGrab()
@@ -210,6 +223,7 @@ def postreposetup_hook(conduit):
     if not loadcache:
         write_timedhosts()
 
+
 def read_timedhosts():
     """
     This function reads the time and hostname from the plugin's cache file and
@@ -331,9 +345,14 @@ class FastestMirror:
         response time.
         """
         self._poll_mirrors()
-        mirrors = [(v, k) for k, v in self.results.items()]
+        if not downgrade_ftp:
+            mirrors = [(v, k) for k, v in self.results.items()]
+        else:
+            # False comes before True
+            mirrors = [(k.startswith("ftp"), v, k) for k, v in
+                       self.results.items()]
         mirrors.sort()
-        return [x[1] for x in mirrors]
+        return [x[-1] for x in mirrors]
 
     def _poll_mirrors(self):
         """
@@ -357,17 +376,13 @@ class FastestMirror:
                 mhost = "127.0.0.1"
             else:
                 mhost = host(mirror)
-            if downgrade_ftp and mirror.startswith("ftp:"):
-                # One less than "dead"
-                self._add_result(mirror, mhost, 99999999998)
-                continue
 
             if mhost in timedhosts:
                 result = timedhosts[mhost]
                 if verbose:
                     print "%s already timed: %s" % (mhost, result)
                 self._add_result(mirror, mhost, result)
-            elif mhost in ("127.0.0.1", "::1", "localhost"):
+            elif mhost in ("127.0.0.1", "::1", "localhost", prefer):
                 self._add_result(mirror, mhost, 0)
             else:
                 # No cached info. so spawn a thread and find the info. out
@@ -458,7 +473,7 @@ class PollThread(threading.Thread):
                 if verbose:
                     print "%s already timed: %s" % (self.host, result)
             else:
-                if self.host in ("127.0.0.1", "::1", "localhost"):
+                if self.host in ("127.0.0.1", "::1", "localhost", prefer):
                     result = 0
                 else:
                     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
