@@ -29,10 +29,13 @@
 # yum --bz  235374 --bz 234688 <cmd>
 # yum --advisory FEDORA-2007-420 --advisory FEDORA-2007-346 <cmd>
 #
-# yum list-security
-# yum list-security bugzillas / bzs
-# yum list-security cves
-# yum list-security security / sec
+# yum list-updateinfo
+# yum list-updateinfo bugzillas / bzs
+# yum list-updateinfo cves
+# yum list-updateinfo security / sec
+# yum list-updateinfo new
+#
+# yum summary-updateinfo
 #
 # yum update-minimal --security
 
@@ -49,6 +52,12 @@ import rpmUtils.miscutils
 
 requires_api_version = '2.5'
 plugin_type = (TYPE_INTERACTIVE,)
+__package_name__ = "yum-plugin-security"
+
+# newpackages is weird, in that we'll never display that because we filter to
+# things relevant to installed pkgs...
+__update_info_types__ = ("security", "bugfix", "enhancement",
+                         "recommended", "newpackage")
 
 def _rpm_tup_vercmp(tup1, tup2):
     """ Compare two "std." tuples, (n, a, e, v, r). """
@@ -118,17 +127,19 @@ def ysp_should_filter_pkg(opts, pkgname, notice, used_map):
         return True
     elif opts.bz and _has_id(used_map, notice['references'],"bugzilla",opts.bz):
         return True
-    elif opts.security:
-        if notice['type'] == 'security':
-            return True
-    elif not (opts.advisory or opts.cve or opts.bz or opts.security or \
-              opts.sec_cmds):
+    # FIXME: Add opts for enhancement/etc.? -- __update_info_types__
+    elif opts.security and notice['type'] == 'security':
+        return True
+    elif opts.bugfixes and notice['type'] == 'bugfix':
+        return True
+    elif not (opts.advisory or opts.cve or opts.bz or
+              opts.security or opts.bugfixes or opts.sec_cmds):
         return True # This is only possible from should_show_pkg
     return False
 
 def ysp_has_info_md(rname, md):
-    if rname == "security":
-        if md['type'] == 'security':
+    if rname in __update_info_types__:
+        if md['type'] == rname:
             return md
     for ref in ysp__safe_refs(md['references']):
         if ref['type'] != rname:
@@ -136,7 +147,7 @@ def ysp_has_info_md(rname, md):
         return md
 
 def ysp_should_show_pkgtup(opts, pkgtup, md_info, used_map, rname=None):
-    """ Do we want to show this package in list-security. """
+    """ Do we want to show this package in list-updateinfo. """
     
     name = pkgtup[0]
     for (pkgtup, notice) in reversed(md_info.get_applicable_notices(pkgtup)):
@@ -173,10 +184,10 @@ def ysp_chk_used_map(used_map, msg):
 
 class SecurityListCommand:
     def getNames(self):
-        return ['list-security', 'list-sec']
+        return ['list-updateinfo', 'list-security', 'list-sec']
 
     def getUsage(self):
-        return "[security|bugzilla|cve] [PACKAGE-wildcard]"
+        return "[security|bugzilla|cve|new-packages] [PACKAGE-wildcard]"
 
     def getSummary(self):
         return "Returns security data for the packages listed, that affects your system"
@@ -184,7 +195,7 @@ class SecurityListCommand:
     def doCheck(self, base, basecmd, extcmds):
         pass
 
-    def show_pkg(self, msg, pkg, notice, disp=None):
+    def show_pkg(self, base, msg, pkg, notice, disp=None):
         # Make the list view much smaller
         # ysp_show_pkg_md_info(pkg, md, msg)
         if disp and ysp_has_info_md(disp, notice):
@@ -192,12 +203,24 @@ class SecurityListCommand:
                 if ref['type'] != disp:
                     continue
                 msg(" %s %-8s %s" % (str(ref['id']), notice['type'], pkg))
+        elif notice['type'] == 'newpackage':
+            print base.fmtKeyValFill("%s: " % pkg.name, base._enc(pkg.summary))
         else:
             msg("%s %-8s %s" % (notice['update_id'], notice['type'], pkg))
 
-    def show_pkg_exit(self):
+    def show_pkg_exit(self, base, md_info):
         pass
-            
+
+    def _get_new_pkgs(self, md_info):
+        for notice in md_info.notices:
+            if notice['type'] != "newpackage":
+                continue
+            for upkg in notice['pkglist']:
+                for pkg in upkg['packages']:
+                    pkgtup = (pkg['name'], pkg['arch'], pkg['epoch'] or '0',
+                              pkg['version'], pkg['release'])
+                    yield (notice, pkgtup)
+
     def doCommand(self, base, basecmd, extcmds):
         self.repos = base.repos
         md_info = ysp_gen_metadata(self.repos.listEnabled())
@@ -225,20 +248,43 @@ class SecurityListCommand:
             
             elif filt_type == "sec":
                 filt_type = "security"
-            elif filt_type == "security":
+            elif filt_type in __update_info_types__:
                 pass
             
             elif filt_type == "cves":
                 filt_type = "cve"
             elif filt_type == "cve":
                 pass
+            elif filt_type == "newpackages":
+                filt_type = "newpackage"
+            elif filt_type == "new-packages":
+                filt_type = "newpackage"
+            elif filt_type == "new":
+                filt_type = "newpackage"
             else:
                 extcmds = [filt_type] + extcmds
                 filt_type = None
             show_type = filt_type
-            if filt_type and filt_type == "security":
+            if filt_type and filt_type in __update_info_types__:
                 show_type = None
-            
+
+        if filt_type == "newpackage":
+            # No filtering here, as we want what isn't installed...
+            done_pkgs = set()
+            for (notice, pkgtup) in sorted(self._get_new_pkgs(md_info),
+                                           key=lambda x: x[1][0]):
+                if extcmds and not _match_sec_cmd(extcmds, pkgtup[0], notice):
+                    continue
+                if pkgtup[0] in done_pkgs:
+                    continue
+                pkgs = base.pkgSack.searchPkgTuple(pkgtup)
+                if not pkgs:
+                    continue
+                done_pkgs.add(pkgs[0].name)
+                self.show_pkg(base, msg, pkgs[0], notice, None)
+            self.show_pkg_exit(base, md_info)
+            return 0, [basecmd + ' new done']
+
         opts.sec_cmds = extcmds
         used_map = ysp_gen_used_map(opts)
         name2tup = _get_name2oldpkgtup(base)
@@ -253,29 +299,71 @@ class SecurityListCommand:
                     d['epoch'] = ''
                 else:
                     d['epoch'] = "%s:" % d['e']
-                self.show_pkg(msg, "%(n)s-%(epoch)s%(v)s-%(r)s.%(a)s" % d,
+                self.show_pkg(base, msg, "%(n)s-%(epoch)s%(v)s-%(r)s.%(a)s" % d,
                               notice, show_type)
         ysp_chk_used_map(used_map, msg)
 
-        self.show_pkg_exit()
+        self.show_pkg_exit(base, md_info)
         return 0, [basecmd + ' done']
             
+
 class SecurityInfoCommand(SecurityListCommand):
     show_pkg_info_done = {}
     def getNames(self):
-        return ['info-security', 'info-sec']
+        return ['info-updateinfo', 'info-security', 'info-sec']
 
-    def show_pkg(self, msg, pkg, notice, disp=None):
+    def show_pkg(self, base, msg, pkg, notice, disp=None):
         if notice['update_id'] in self.show_pkg_info_done:
             return
         self.show_pkg_info_done[notice['update_id']] = True
         # Python-2.4.* doesn't understand str(x) returning unicode *sigh*
         obj = notice.__str__()
         msg(obj)
-    
-    def show_pkg_exit(self):
+
+    def show_pkg_exit(self, base, md_info):
         self.show_pkg_info_done = {}
 
+
+class SecuritySummaryCommand(SecurityListCommand):
+    show_pkg_info_done = {}
+    def getNames(self):
+        return ['summary-updateinfo']
+
+    def show_pkg(self, base, msg, pkg, notice, disp=None):
+        if notice['update_id'] in self.show_pkg_info_done:
+            return
+        self.show_pkg_info_done[notice['update_id']] = notice
+
+    def show_pkg_exit(self, base, md_info):
+        def _msg(x):
+            print x
+        counts = {}
+        for notice in self.show_pkg_info_done.values():
+            counts[notice['type']] = counts.get(notice['type'], 0) + 1
+        maxsize = 0
+        for T in ('newpackage', 'security', 'bugfix', 'enhancement'):
+            if T not in counts:
+                continue
+            size = len(str(counts[T]))
+            if maxsize < size:
+                maxsize = size
+        if not maxsize:
+            _check_running_kernel(base, md_info, _msg)
+            return
+
+        outT = {'newpackage' : 'New Package',
+                'security' : 'Security',
+                'bugfix' : 'Bugfix',
+                'enhancement' : 'Enhancement'}
+        print "Updates Info Summary:"
+        for T in ('newpackage', 'security', 'bugfix', 'enhancement'):
+            if T not in counts:
+                continue
+            print "    %*u %s notice(s)" % (maxsize, counts[T], outT[T])
+        _check_running_kernel(base, md_info, _msg)
+        self.show_pkg_info_done = {}
+
+
 # "Borrowed" from yumcommands.py
 def yumcommands_checkRootUID(base):
     """
@@ -334,12 +422,15 @@ class SecurityUpdateCommand:
         yumcommands_checkGPGKey(base)
 
     def doCommand(self, base, basecmd, extcmds):
+        if hasattr(base, 'run_with_package_names'):
+            base.run_with_package_names.add(__package_name__)
         md_info       = ysp_gen_metadata(base.repos.listEnabled())
         opts          = base.plugins.cmdline[0]
         opts.sec_cmds = []
         used_map      = ysp_gen_used_map(opts)
 
-        ndata = not (opts.security or opts.advisory or opts.bz or opts.cve)
+        ndata = not (opts.security or opts.bugfixes or
+                     opts.advisory or opts.bz or opts.cve)
 
         # NOTE: Not doing obsoletes processing atm. ... maybe we should? --
         # Also worth pointing out we don't go backwards for obsoletes in the:
@@ -377,24 +468,26 @@ def config_hook(conduit):
     '''
     Yum Plugin Config Hook: 
     Setup the option parser with the '--advisory', '--bz', '--cve', and
-    '--security' command line options. And the 'list-security',
-    'info-security', and 'update-minimal' commands.
+    '--security' command line options. And the 'list-updateinfo',
+    'info-updateinfo', and 'update-minimal' commands.
     '''
 
     parser = conduit.getOptParser()
     if not parser:
         return
 
+    if hasattr(parser, 'plugin_option_group'):
+        parser = parser.plugin_option_group
+
     conduit.registerCommand(SecurityListCommand())
     conduit.registerCommand(SecurityInfoCommand())
+    conduit.registerCommand(SecuritySummaryCommand())
     conduit.registerCommand(SecurityUpdateCommand())
-    parser.values.advisory = []
-    parser.values.cve      = []
-    parser.values.bz       = []
-    parser.values.security = False
     def osec(opt, key, val, parser):
          # CVE is a subset of --security on RHEL, but not on Fedora
         parser.values.security = True
+    def obug(opt, key, val, parser):
+        parser.values.bugfixes = True
     def ocve(opt, key, val, parser):
         parser.values.cve.append(val)
     def obz(opt, key, val, parser):
@@ -405,6 +498,9 @@ def config_hook(conduit):
     parser.add_option('--security', action="callback",
                       callback=osec, dest='security', default=False,
                       help='Include security relevant packages')
+    parser.add_option('--bugfixes', action="callback",
+                      callback=obug, dest='bugfixes', default=False,
+                      help='Include bugfix relevant packages')
     parser.add_option('--cve', action="callback", type="string",
                       callback=ocve, dest='cve', default=[],
                       help='Include packages needed to fix the given CVE')
@@ -438,13 +534,14 @@ def ysp_check_func_enter(conduit):
     
     opts, args = conduit.getCmdLine()
 
-    ndata = not (opts.security or opts.advisory or opts.bz or opts.cve)
+    ndata = not (opts.security or opts.bugfixes or
+                 opts.advisory or opts.bz or opts.cve)
     
     ret = None
     if len(args) >= 2:
-        if ((args[0] == "list") and (args[1] == "updates")):
+        if ((args[0] == "list") and (args[1] in ("obsoletes", "updates"))):
             ret = {"skip": ndata, "list_cmd": True}
-        if ((args[0] == "info") and (args[1] == "updates")):
+        if ((args[0] == "info") and (args[1] in ("obsoletes", "updates"))):
             ret = {"skip": ndata, "list_cmd": True}
     if len(args):
 
@@ -456,9 +553,9 @@ def ysp_check_func_enter(conduit):
             ret = {"skip": ndata, "list_cmd": True}
         if (args[0] in ["update", "upgrade"]):
             ret = {"skip": ndata, "list_cmd": False}
-        if (args[0] == "list-sec") or (args[0] == "list-security"):
+        if (args[0] in ("list-sec", "list-security", 'list-updateinfo')):
             return (opts, {"skip": True, "list_cmd": True})
-        if (args[0] == "info-sec") or (args[0] == "info-security"):
+        if (args[0] in ("info-sec", "info-security", 'info-updateinfo')):
             return (opts, {"skip": True, "list_cmd": True})
 
     if ret:
@@ -483,6 +580,8 @@ def exclude_hook(conduit):
     if not info["list_cmd"]:
         return
     
+    if hasattr(conduit, 'registerPackageName'):
+        conduit.registerPackageName(__package_name__)
     conduit.info(2, 'Limiting package lists to security relevant ones')
     
     md_info = ysp_gen_metadata(conduit.getRepos().listEnabled())
@@ -504,6 +603,9 @@ def exclude_hook(conduit):
     # for the list of update packages, which is all we care about.    
     upds = conduit._base.doPackageLists(pkgnarrow='updates')
     pkgs = upds.updates
+    # In theory we don't need to do this in some cases, but meh.
+    upds = conduit._base.doPackageLists(pkgnarrow='obsoletes')
+    pkgs += upds.obsoletes
 
     name2tup = _get_name2oldpkgtup(conduit._base)
     
@@ -520,10 +622,38 @@ def exclude_hook(conduit):
 
     ysp_chk_used_map(used_map, lambda x: conduit.error(2, x))
     if cnt:
-        conduit.info(2, 'Needed %d of %d packages, for security' % (cnt, tot))
+        conduit.info(2, '%d package(s) needed for security, out of %d available' % (cnt, tot))
     else:
-        conduit.info(2, 'No packages needed, for security, %d available' % tot)
-            
+        conduit.info(2, 'No packages needed for security; %d packages available' % tot)
+
+    _check_running_kernel(conduit._base, md_info, lambda x: conduit.info(2, x))
+
+def _check_running_kernel(yb, md_info, msg):
+    if not hasattr(yum.misc, 'get_running_kernel_pkgtup'):
+        return # Back compat.
+
+    kern_pkgtup = yum.misc.get_running_kernel_pkgtup(yb.ts)
+    if kern_pkgtup[0] is None:
+        return
+
+    found_sec = False
+    for (pkgtup, notice) in md_info.get_applicable_notices(kern_pkgtup):
+        if found_sec or notice['type'] != 'security':
+            continue
+        found_sec = True
+        ipkg = yb.rpmdb.searchPkgTuple(pkgtup)
+        if not ipkg:
+            continue # Not installed
+        ipkg = ipkg[0]
+        rpkg = '%s-%s:%s-%s.%s' % (kern_pkgtup[0], kern_pkgtup[2],
+                                   kern_pkgtup[3], kern_pkgtup[4],
+                                   kern_pkgtup[1])
+
+        msg('Security: %s is an installed security update' % ipkg)
+        msg('Security: %s is the currently running version' % rpkg)
+        break
+
+
 def preresolve_hook(conduit):
     '''
     Yum Plugin PreResolve Hook:
@@ -537,6 +667,8 @@ def preresolve_hook(conduit):
     if info["list_cmd"]:
         return
     
+    if hasattr(conduit, 'registerPackageName'):
+        conduit.registerPackageName(__package_name__)
     conduit.info(2, 'Limiting packages to security relevant ones')
 
     md_info = ysp_gen_metadata(conduit.getRepos().listEnabled())
@@ -606,9 +738,9 @@ def preresolve_hook(conduit):
     ysp_chk_used_map(used_map, lambda x: conduit.error(2, x))
     
     if cnt:
-        conduit.info(2, 'Needed %d (+%d related) of %d packages (so far), for security' % (scnt, cnt - scnt, tot))
+        conduit.info(2, '%d package(s) needed (+%d related) for security, out of %d available' % (scnt, cnt - scnt, tot))
     else:
-        conduit.info(2, 'No packages needed, for security, of %d packages (so far)' % tot)
+        conduit.info(2, 'No packages needed for security; %d packages available' % tot)
 
 if __name__ == '__main__':
     print "This is a plugin that is supposed to run from inside YUM"
