@@ -26,6 +26,7 @@ import time
 import os
 import os.path
 import exceptions
+import urlparse
 
 from optparse import OptionParser
 
@@ -35,6 +36,7 @@ import yum.misc as misc
 import yum.config
 import yum.Errors
 import yum.packages
+from yum.i18n import to_unicode
 from rpmUtils.arch import getArchList
 from rpmUtils.miscutils import formatRequire
 import output
@@ -76,10 +78,10 @@ def sec2isodate(timestr):
     return time.strftime("%F %T", time.gmtime(int(timestr)))
 
 def sec2date(timestr):
-    return time.ctime(int(timestr))
+    return to_unicode(time.ctime(int(timestr)))
 
 def sec2day(timestr):
-    return time.strftime("%a %b %d %Y", time.gmtime(int(timestr)))
+    return to_unicode(time.strftime("%a %b %d %Y", time.gmtime(int(timestr))))
 
 def _size2val(size, off, ui):
     size = float(size)
@@ -228,6 +230,18 @@ class pkgQuery:
     def fmt_envra(self, **kw):
         return "%(envr)s.%(arch)s" % self
 
+    def fmt_location(self, **kw):
+        loc = ''
+        repo = self.pkg.repo
+        if self['basepath']:
+            loc = "%(basepath)s/%(relativepath)s" % self
+        else:
+            repourl = repo.urls[0]
+            if repourl[-1] != '/':
+                repourl = repourl + '/'
+            loc = urlparse.urljoin(repourl, self['relativepath'])
+        return loc
+
 class repoPkgQuery(pkgQuery):
     """
     I wrap a query of a non-installed package available in the repository.
@@ -251,10 +265,10 @@ class repoPkgQuery(pkgQuery):
     def files(self, **kw):
         fdict = {}
         for ftype in self.pkg.returnFileTypes():
-            for file in self.pkg.returnFileEntries(ftype):
+            for fn in self.pkg.returnFileEntries(ftype):
                 # workaround for yum returning double leading slashes on some 
                 # directories - posix allows that but it looks a bit odd
-                fdict[os.path.normpath('//%s' % file)] = None
+                fdict[os.path.normpath('//%s' % fn)] = None
         files = fdict.keys()
         files.sort()
         return files
@@ -262,7 +276,9 @@ class repoPkgQuery(pkgQuery):
     def fmt_changelog(self, **kw):
         changelog = []
         for date, author, message in self.pkg.returnChangelog():
-            changelog.append("* %s %s\n%s\n" % (sec2day(date), author, message))
+            changelog.append("* %s %s\n%s\n" % (sec2day(date),
+                                                to_unicode(author),
+                                                to_unicode(message)))
         return "\n".join(changelog)
 
 class instPkgQuery(pkgQuery):
@@ -352,7 +368,7 @@ class groupQuery:
             elif t == "all":
                 pkgs.extend(self.group.packages)
             else:
-                raise "Unknown group package type %s" % t
+                raise queryError("Unknown group package type %s" % t)
             
         return pkgs
         
@@ -371,6 +387,10 @@ class YumBaseQuery(yum.YumBase):
         """
         yum.YumBase.__init__(self)
         self.logger = logging.getLogger("yum.verbose.repoquery")
+        console_stderr = logging.StreamHandler(sys.stderr)
+        console_stderr.setFormatter(logging.Formatter("%(message)s"))
+        self.logger.propagate = False
+        self.logger.addHandler(console_stderr)
         self.options = options
         self.pkgops = pkgops
         self.sackops = sackops
@@ -387,6 +407,8 @@ class YumBaseQuery(yum.YumBase):
         qpkgs = []
         for pkg in pkgs:
             if isinstance(pkg, yum.packages.YumInstalledPackage):
+                if self.options.pkgnarrow not in ('all', 'installed', 'extras'):
+                    continue
                 qpkg = instPkgQuery(pkg, qf)
             else:
                 qpkg = repoPkgQuery(pkg, qf)
@@ -403,24 +425,25 @@ class YumBaseQuery(yum.YumBase):
         """
         pkgs = []
         try:
-            exact, match, unmatch = yum.packages.parsePackages(self.returnPkgList(), [name], casematch=1)
-            pkgs = exact + match
+            pkgs = self.returnPkgList(patterns=[name])
         except yum.Errors.PackageSackError, err:
             self.logger.error(err)
         return self.queryPkgFactory(pkgs)
 
-    def returnPkgList(self):
+    def returnPkgList(self, **kwargs):
         pkgs = []
         if self.options.pkgnarrow == "repos":
             # self.pkgSack is a yum.packageSack.MetaSack
             if self.conf.showdupesfromrepos:
-                pkgs = self.pkgSack.returnPackages()
+                pkgs = self.pkgSack.returnPackages(**kwargs)
             else:
-                pkgs = self.pkgSack.returnNewestByNameArch()
-
+                try:
+                    pkgs = self.pkgSack.returnNewestByNameArch(**kwargs)
+                except yum.Errors.PackageSackError:
+                    pkgs = []
         else:
             what = self.options.pkgnarrow
-            ygh = self.doPackageLists(what)
+            ygh = self.doPackageLists(what, **kwargs)
 
             if what == "all":
                 pkgs = ygh.available + ygh.installed
@@ -462,14 +485,7 @@ class YumBaseQuery(yum.YumBase):
         return grps
                     
     def matchPkgs(self, items):
-        pkgs = []
-        notfound = {}
-        
-        exact, match, unmatch = yum.packages.parsePackages(self.returnPkgList(),
-                                           items, casematch=1)
-        pkgs = exact + match
-        notfound = unmatch
-
+        pkgs = self.returnPkgList(patterns=items)
         return self.queryPkgFactory(pkgs)
 
     def matchSrcPkgs(self, items):
@@ -494,13 +510,17 @@ class YumBaseQuery(yum.YumBase):
         for pkg in pkgs:
             for oper in self.pkgops:
                 try:
-                    print pkg.doQuery(oper)
+                    out = pkg.doQuery(oper)
+                    if out:
+                        print to_unicode(out)
                 except queryError, e:
                     self.logger.error( e.msg)
         for prco in items:
             for oper in self.sackops:
                 try:
-                    for p in self.doQuery(oper, prco): print p
+                    for p in self.doQuery(oper, prco): 
+                        if p:
+                            print p
                 except queryError, e:
                     self.logger.error( e.msg)
 
@@ -570,7 +590,10 @@ class YumBaseQuery(yum.YumBase):
         loc = []
         for pkg in self.returnByName(name):
             repo = self.repos.getRepo(pkg['repoid'])
-            loc.append("%s/%s" % (repo.urls[0], pkg['relativepath']))
+            if pkg['basepath']:
+                loc.append("%s/%s" % (pkg['basepath'], pkg['relativepath']))
+            else:
+                loc.append("%s/%s" % (repo.urls[0], pkg['relativepath']))
         return loc
 
 def main(args):
@@ -644,6 +667,8 @@ def main(args):
     # other opts
     parser.add_option("--archlist",
                       help="only query packages of certain architecture(s)")
+    parser.add_option("--releasever", default=None,
+                      help="set value of $releasever in yum config and repo files")
     parser.add_option("--pkgnarrow", default="repos",
                       help="limit query to installed / available / recent / updates / extras / available + installed / repository (default) packages")
     parser.add_option("--show-duplicates", action="store_true",
@@ -654,7 +679,7 @@ def main(args):
     parser.add_option("--repoid", action="append",
                       help="specify repoids to query, can be specified multiple times (default is all enabled)")
     parser.add_option("--enablerepo", action="append", dest="enablerepos",
-                      help="specify repoids to query, can be specified multiple times (default is all enabled)")
+                      help="specify additional repoids to query, can be specified multiple times")
     parser.add_option("--disablerepo", action="append", dest="disablerepos",
                       help="specify repoids to disable, can be specified multiple times")                      
     parser.add_option("--repofrompath", action="append",
@@ -671,7 +696,8 @@ def main(args):
                       help="use private cache (default when used as non-root)")
     parser.add_option("--querytags", action="store_true",
                       help="list available tags in queryformat queries")
-    parser.add_option("-c", dest="conffile", help="config file location")
+    parser.add_option("-c", "--config", dest="conffile",
+                      help="config file location")
 
     (opts, regexs) = parser.parse_args()
 
@@ -685,7 +711,7 @@ def main(args):
         if opts.all:
             regexs = ['*']
         else:
-            parser.print_help()
+            print parser.format_help()
             sys.exit(1)
 
     pkgops = []
@@ -732,7 +758,7 @@ def main(args):
     if opts.file:
         sackops.append("whatprovides")
     if opts.location:
-        sackops.append("location")
+        pkgops.append("location")
     if opts.groupmember:
         sackops.append("groupmember")
         needgroup = 1
@@ -746,6 +772,7 @@ def main(args):
 
     # silence initialisation junk from modules etc unless verbose mode
     initnoise = (not opts.quiet) * 2
+    repoq.releasever = opts.releasever
     if opts.conffile:
         repoq.doConfigSetup(fn=opts.conffile, debuglevel=initnoise, init_plugins=opts.plugins)
     else:
@@ -760,14 +787,10 @@ def main(args):
             else:
                 baseurl = repopath
                 
-            repopath = os.path.normpath(repopath)
-            newrepo = yum.yumRepo.YumRepository(repoid)
-            newrepo.name = repopath
-            newrepo.baseurl = baseurl
-            newrepo.basecachedir = repoq.conf.cachedir
-            repoq.repos.add(newrepo)
-            repoq.repos.enableRepo(newrepo.id)
-            repoq.logger.info( "Added %s repo from %s" % (repoid,repopath))
+            repoq.add_enable_repo(repoid, baseurls=[baseurl], 
+                    basecachedir=repoq.conf.cachedir)
+            if not opts.quiet:
+                repoq.logger.info( "Added %s repo from %s" % (repoid,repopath))
 
         
     # Show what is going on, if --quiet is not set.
@@ -778,13 +801,9 @@ def main(args):
         freport = ( yumout.failureReport, (), {} )
         repoq.repos.setFailureCallback( freport )       
     
-    if os.geteuid() != 0 or opts.tempcache:
-        cachedir = misc.getCacheDir()
-        if cachedir is None:
-            repoq.logger.error("Error: Could not make cachedir, exiting")
-            sys.exit(50)
-        repoq.repos.setCacheDir(cachedir)
-        repoq.conf.cache = 0 # yum set cache=1, if uid != 0
+    if not repoq.setCacheDir(opts.tempcache):
+        repoq.logger.error("Error: Could not make cachedir, exiting")
+        sys.exit(50)
 
     if opts.cache:
         repoq.conf.cache = True
@@ -807,9 +826,9 @@ def main(args):
                 repo.disable()
 
     if opts.enablerepos:    
-            for repo_match in opts.enablerepos:
-                for repo in repoq.repos.findRepos(repo_match):
-                    repo.enable()
+        for repo_match in opts.enablerepos:
+            for repo in repoq.repos.findRepos(repo_match):
+                repo.enable()
 
     try:
         repoq.doRepoSetup()
@@ -837,8 +856,8 @@ def main(args):
             repoq.repos.populateSack(mdtype='otherdata')
         if needgroup:
             repoq.doGroupSetup()
-    except yum.Errors.RepoError, e:
-        repoq.logger.error( e)
+    except (yum.Errors.RepoError, yum.Errors.GroupsError), e:
+        repoq.logger.error(e)
         sys.exit(1)
 
     repoq.runQuery(regexs)
